@@ -22,7 +22,7 @@ use Log::Scrubber qw(disable $SCRUBBER scrubber :Carp scrubber_add_scrubber);
 @ISA     = qw(Business::OnlinePayment::HTTPS);
 $me      = 'Business::OnlinePayment::Litle';
 $DEBUG   = 0;
-our $VERSION = '0.957'; # VERSION
+our $VERSION = '0.958'; # VERSION
 
 # PODNAME: Business::OnlinePayment::Litle
 
@@ -129,8 +129,8 @@ sub set_defaults {
         }
     }
 
-    $self->api_version('8.1')                   unless $self->api_version;
-    $self->batch_api_version('8.1')             unless $self->batch_api_version;
+    $self->api_version('11.0')                   unless $self->api_version;
+    $self->batch_api_version('11.0')             unless $self->batch_api_version;
     $self->chargeback_api_version('2.2')        unless $self->chargeback_api_version;
     $self->xmlns('http://www.litle.com/schema') unless $self->xmlns;
 }
@@ -145,11 +145,11 @@ sub test_transaction {
     $self->{'test_transaction'} = 'sandbox';
         $self->verify_SSL(0);
 
-        $self->server('www.testlitle.com');
+        $self->server('www.testvantivcnp.com');
         $self->port('443');
         $self->path('/sandbox/communicator/online');
 
-        $self->chargeback_server('service-postlive.litle.com'); # no sandbox exists, so fallback to certify
+        $self->chargeback_server('services.vantivpostlive.com'); # no sandbox exists, so fallback to certify
         $self->chargeback_port('443');
         $self->chargeback_path('/services/communicator/chargebacks/webCommunicator');
     } elsif (lc($testMode) eq 'localhost') {
@@ -216,10 +216,10 @@ sub map_fields {
         'auth reversal'        => 'authReversal',
         'account update'       => 'accountUpdate',
         'tokenize'             => 'registerTokenRequest',
+        'force capture'        => 'force_capture',
 
         # AVS ONLY
         # Capture Given
-        # Force Capture
         #
     );
     $content->{'TransactionType'} = $actions{$action} || $action;
@@ -415,6 +415,21 @@ sub map_request {
       #warn "$trunc->[0] => ".($content->{ $trunc->[0] }||'')."\n" if $DEBUG;
     }
 
+    tie my %customer_info, 'Tie::IxHash', $self->_revmap_fields(
+        content                  => $content,
+        ssn                      => 'ssn',
+        dob                      => 'dob',
+        customerRegistrationDate => 'registration_date',
+        customerType             => 'customer_type',
+        incomeAmount             => 'income_amount',
+        incomeCurrency           => 'income_currency',
+        employerName             => 'employer_name',
+        customerWorkTelephone    => 'work_phone',
+        residenceStatus          => 'residence_status',
+        yearsAtResidence         => 'residence_years',
+        yearsAtEmployer          => 'employer_years',
+    );
+
     tie my %billToAddress, 'Tie::IxHash', $self->_revmap_fields(
         content      => $content,
         name         => 'name',
@@ -431,13 +446,15 @@ sub map_request {
     tie my %shipToAddress, 'Tie::IxHash', $self->_revmap_fields(
         content      => $content,
         name         => 'ship_name',
-        email        => 'ship_email',
         addressLine1 => 'ship_address',
+        addressLine2 => 'ship_address2',
+        addressLine3 => 'ship_address3',
         city         => 'ship_city',
         state        => 'ship_state',
         zip          => 'ship_zip',
         country      => 'ship_country'
         , #TODO: will require validation to the spec, this field wont' work as is
+        email        => 'ship_email',
         phone => 'ship_phone',
     );
 
@@ -449,9 +466,10 @@ sub map_request {
 
     tie my %custombilling, 'Tie::IxHash',
       $self->_revmap_fields(
-        content      => $content,
+        content    => $content,
         phone      => 'company_phone',
         descriptor => 'description',
+        #url        => 'url',
       );
 
     ## loop through product list and generate lineItemData for each
@@ -497,26 +515,69 @@ sub map_request {
       }
     }
 
+    tie my %filtering, 'Tie::IxHash', $self->_revmap_fields(
+        content                  => $content,
+        prepaid       => 'filter_prepaid',
+        international => 'filter_international',
+        chargeback    => 'filter_chargeback',
+    );
+
+    tie my %healthcaresub, 'Tie::IxHash', $self->_revmap_fields(
+        content                  => $content,
+        totalHealthcareAmount => 'amount_healthcare',
+        RxAmount          => 'amount_medications',
+        visionAmount      => 'amount_vision',
+        clinicOtherAmount => 'amount_clinic',
+        dentalAmount      => 'amount_dental',
+    );
+
+    tie my %healthcare, 'Tie::IxHash', $self->_revmap_fields(
+        content                  => $content,
+        healthcareAmounts => \%healthcaresub,
+        IIASFlag          => 'healthcare_flag',
+    );
+
+    tie my %amexaggregator, 'Tie::IxHash', $self->_revmap_fields(
+        content                  => $content,
+        sellerId                   => 'amex_seller_id',
+        sellerMerchantCategoryCode => 'amex_merch_code',
+    );
+
+    tie my %detailtax, 'Tie::IxHash', $self->_revmap_fields(
+        content                  => $content,
+        taxIncludedInTotal => 'tax_in_total',
+        taxAmount          => 'tax_amount',
+        taxRate            => 'tax_rate',
+        taxTypeIdentifier  => 'tax_type',
+        cardAcceptorTaxId  => 'tax_id',
+    );
     #
     #
     tie my %enhanceddata, 'Tie::IxHash', $self->_revmap_fields(
         content                => $content,
         customerReference      => 'po_number',
         salesTax               => 'salestax',
+        deliveryType           => 'deliverytype',
+        taxExempt              => 'tax_exempt',
         discountAmount         => 'discount',
         shippingAmount         => 'shipping',
         dutyAmount             => 'duty',
+        shipFromPostalCode     => 'company_zip',
+        destinationPostalCode  => 'ship_zip',
+        destinationCountryCode => 'ship_country',
         invoiceReferenceNumber => 'invoice_number_length_15',
         orderDate              => 'orderdate',
+        detailTax              => \%detailtax,
         lineItemData           => \@products,
     );
 
     tie my %card, 'Tie::IxHash', $self->_revmap_fields(
-        content            => $content,
-        type               => 'card_type',
-        number             => 'card_number',
-        expDate            => 'expiration',
-        cardValidationNum  => 'cvv2',
+        content           => $content,
+        type              => 'card_type',
+        number            => 'card_number',
+        expDate           => 'expiration',
+        cardValidationNum => 'cvv2',
+        pin               => 'pin',
     );
 
     tie my %token, 'Tie::IxHash', $self->_revmap_fields(
@@ -526,11 +587,37 @@ sub map_request {
         cardValidationNum  => 'cvv2',
     );
 
+    tie my %sepadirect, 'Tie::IxHash', $self->_revmap_fields(
+        content              => $content,
+        mandateProvider      => 'sepa_mandate_provider',
+        sequenceType         => 'sepa_sequence_type',
+        mandateReference     => 'sepa_mandate_reference',
+        mandateUrl           => 'sepa_mandate_url',
+        mandateSignatureDate => 'sepa_mandate_signature_date',
+        iban                 => 'sepa_iban',
+        preferredLanguage    => 'sepa_language',
+    );
+    
+    tie my %ideal, 'Tie::IxHash', $self->_revmap_fields(
+        content           => $content,
+        preferredLanguage => 'ideal_language',
+    );
+
     tie my %processing, 'Tie::IxHash', $self->_revmap_fields(
         content               => $content,
         bypassVelocityCheck   => 'velocity_check',
     );
 
+    tie my %pos, 'Tie::IxHash', $self->_revmap_fields(
+        content                  => $content,
+        capability   => 'pos_capability',
+        entryMode    => 'pos_entry_mode',
+        cardholderId => 'pos_cardholder_id',
+        terminalId   => 'pos_terminal_id',
+        catLevel     => 'pos_cat_level',
+        #For CAT (Cardholder Activated Terminal) transactions, the capability element must be set to magstripe, the cardholderId element must be set to nopin, and the catLevel element must be set to self service.
+    );
+
     tie my %cardholderauth, 'Tie::IxHash',
       $self->_revmap_fields(
         content                     => $content,
@@ -554,64 +641,156 @@ sub map_request {
         recycleId    => 'recycle_id',
       );
 
+    tie my %recurringRequest, 'Tie::IxHash',
+      $self->_revmap_fields(
+        content          => $content,
+        planCode         => 'recurring_plan_code',
+        numberOfPayments => 'recurring_number_of_payments',
+        startDate        => 'recurring_start_date',
+        amount           => 'recurring_amount',
+      );
+    
+      tie my %advancedfraud, 'Tie::IxHash',
+      $self->_revmap_fields(
+        content               => $content,
+        threatMetrixSessionId => 'threatMetrixSessionId',
+        customAttribute1      => 'advanced_fraud_customAttribute1',
+        customAttribute2      => 'advanced_fraud_customAttribute2',
+        customAttribute3      => 'advanced_fraud_customAttribute3',
+        customAttribute4      => 'advanced_fraud_customAttribute4',
+        customAttribute5      => 'advanced_fraud_customAttribute5',
+      );
+
+    tie my %wallet, 'Tie::IxHash',
+      $self->_revmap_fields(
+        content            => $content,
+        walletSourceType   => 'wallet_source_type',
+        walletSourceTypeId => 'wallet_source_type_id',
+      );
+
     my %req;
 
     if ( $action eq 'registerTokenRequest' ) {
         croak 'missing card_number' if length($content->{'card_number'} || '') == 0;
         tie %req, 'Tie::IxHash', $self->_revmap_fields(
-            content       => $content,
-            orderId       => 'invoice_number',
-            accountNumber => 'card_number',
+            content                      => $content,
+            orderId                      => 'invoice_number',
+            accountNumber                => 'card_number',
         );
     }
     elsif ( $action eq 'sale' ) {
         croak 'missing card_token or card_number' if length($content->{'card_number'} || $content->{'card_token'} || '') == 0;
         tie %req, 'Tie::IxHash', $self->_revmap_fields(
-            content       => $content,
-            orderId       => 'invoice_number',
-            amount        => 'amount',
-            orderSource   => 'orderSource',
-            billToAddress => \%billToAddress,
-            card          => $content->{'card_number'} ? \%card : {},
-            token         => $content->{'card_token'} ? \%token : {},
-
-            #cardholderAuthentication    =>  \%cardholderauth,
-            customBilling => \%custombilling,
-            enhancedData  => \%enhanceddata,
-            processingInstructions  =>  \%processing,
-            allowPartialAuth => 'partial_auth',
-            merchantData => \%merchantdata,
-            recyclingRequest => \%recyclingrequest,
+            content                      => $content,
+            orderId                      => 'invoice_number',
+            amount                       => 'amount',
+            secondaryAmount              => 'secondary_amount',
+            orderSource                  => 'orderSource',
+            customerInfo                 => \%customer_info, # PP only
+            billToAddress                => \%billToAddress,
+            shipToAddress                => \%shipToAddress,
+            card                         => $content->{'card_number'} ? \%card : {},
+            token                        => $content->{'card_token'} ? \%token : {},
+            #[<card>|<paypage>|<token>|<paypal>|<mpos>|<applepay>|
+            #<sepaDirectDebit>|<ideal>] (Choice)
+            sepaDirectDebit              => \%sepadirect,
+            ideal                        => \%ideal,
+            cardholderAuthentication     => \%cardholderauth,
+            customBilling                => \%custombilling,
+            taxType                      => 'tax_type',      # payment|fee
+            enhancedData                 => \%enhanceddata,
+            processingInstructions       => \%processing,
+            amexAggregatorData           => \%amexaggregator,
+            allowPartialAuth             => 'partial_auth',
+            healthcareIIAS               => \%healthcare,
+            filtering                    => \%filtering,
+            merchantData                 => \%merchantdata,
+            recyclingRequest             => \%recyclingrequest,
+            fraudFilterOverride          => 'filter_fraud_override',
+            recurringRequest             => \%recurringRequest,
+            debtRepayment                => 'debt_repayment',
+            advancedFraudChecks          => \%advancedfraud,
+            wallet                       => \%wallet,
+            processingType               => 'processing_type',
+            originalNetworkTransactionId => 'original_network_transaction_id',
+            originalTransactionAmount    => 'original_transaction_amount',
         );
     }
     elsif ( $action eq 'authorization' ) {
         croak 'missing card_token or card_number' if length($content->{'card_number'} || $content->{'card_token'} || '') == 0;
         tie %req, 'Tie::IxHash', $self->_revmap_fields(
-            content       => $content,
-            orderId       => 'invoice_number',
-            amount        => 'amount',
-            orderSource   => 'orderSource',
-            billToAddress => \%billToAddress,
-            card          => $content->{'card_number'} ? \%card : {},
-            token         => $content->{'card_token'} ? \%token : {},
+            content                      => $content,
+            orderId                      => 'invoice_number',
+            amount                       => 'amount',
+            secondaryAmount              => 'secondary_amount',
+            orderSource                  => 'orderSource',
+            customerInfo                 => \%customer_info, #  PP only
+            billToAddress                => \%billToAddress,
+            shipToAddress                => \%shipToAddress,
+            card                         => $content->{'card_number'} ? \%card : {},
+            token                        => $content->{'card_token'} ? \%token : {},
+
+            cardholderAuthentication     => \%cardholderauth,
+            processingInstructions       => \%processing,
+            pos                          => \%pos,
+            customBilling                => \%custombilling,
+            taxType                      => 'tax_type', # payment|fee
+            enhancedData                 => \%enhanceddata,
+            amexAggregatorData           => \%amexaggregator,
+            allowPartialAuth             => 'partial_auth',
+            healthcareIIAS               => \%healthcare,
+            filtering                    => \%filtering,
+            merchantData                 => \%merchantdata,
+            recyclingRequest             => \%recyclingrequest,
+            fraudFilterOverride          => 'filter_fraud_override',
+            recurringRequest             => \%recurringRequest,
+            debtRepayment                => 'debt_repayment',
+            advancedFraudChecks          => \%advancedfraud,
+            wallet                       => \%wallet,
+            processingType               => 'processing_type',
+            originalNetworkTransactionId => 'original_network_transaction_id',
+            originalTransactionAmount    => 'original_transaction_amount',
 
-            #cardholderAuthentication    =>  \%cardholderauth,
-            processingInstructions  =>  \%processing,
-            customBilling => \%custombilling,
-            allowPartialAuth => 'partial_auth',
-            merchantData     => \%merchantdata,
-            recyclingRequest => \%recyclingrequest,
         );
     }
     elsif ( $action eq 'capture' ) {
         push @required_fields, qw( order_number amount );
         tie %req, 'Tie::IxHash',
           $self->_revmap_fields(
-            content      => $content,
-            litleTxnId   => 'order_number',
-            amount       => 'amount',
-            enhancedData => \%enhanceddata,
+              # partial is an element of the start tag, so located in the header
+            content                => $content,
+            litleTxnId             => 'order_number',
+            amount                 => 'amount',
+            surchargeAmount        => 'surcharge_amount',
+            enhancedData           => \%enhanceddata,
             processingInstructions => \%processing,
+            payPalOrderComplete    => 'paypal_order_complete',
+            pin                    => 'pin',
+          );
+    }
+    elsif ( $action eq 'force_capture' ) {
+        ## ARE YOU SURE YOU WANT TO DO THIS?
+        # Seriously, force captures are like running up the pirate flag, check with your Vantiv rep
+        push @required_fields, qw( order_number amount );
+        tie %req, 'Tie::IxHash',
+          $self->_revmap_fields(
+              # partial is an element of the start tag, so located in the header
+            content                => $content,
+            litleTxnId             => 'order_number',
+            amount                 => 'amount',
+            secondaryAmount        => 'secondary_amount',
+            orderSource            => 'orderSource',
+            billToAddress          => \%billToAddress,
+            card                   => $content->{'card_number'} ? \%card : {},
+            token                  => $content->{'card_token'} ? \%token : {},
+            customBilling          => \%custombilling,
+            taxType                => 'tax_type',      # payment|fee
+            enhancedData           => \%enhanceddata,
+            processingInstructions => \%processing,
+            amexAggregatorData     => \%amexaggregator,
+            merchantData           => \%merchantdata,
+            debtRepayment          => 'debt_repayment',
+            processingType         => 'processing_type',
           );
     }
     elsif ( $action eq 'credit' ) {
@@ -620,11 +799,14 @@ sub map_request {
        if( $content->{'order_number'} ){
           push @required_fields, qw( order_number amount );
           tie %req, 'Tie::IxHash', $self->_revmap_fields(
-              content       => $content,
-              litleTxnId    => 'order_number',
-              amount        => 'amount',
-              customBilling => \%custombilling,
+              content                => $content,
+              litleTxnId             => 'order_number',
+              amount                 => 'amount',
+              secondaryAmount        => 'secondary_amount',
+              customBilling          => \%custombilling,
+              enhancedData           => \%enhanceddata,
               processingInstructions => \%processing,
+              actionReason           => 'action_reason', #  ENUM(SUSPECT_FRAUD) only option atm
           );
         }
        # ELSE it's an unlinked, which requires different data
@@ -632,15 +814,21 @@ sub map_request {
           croak 'missing card_token or card_number' if length($content->{'card_number'} || $content->{'card_token'} || '') == 0;
           push @required_fields, qw( invoice_number amount );
           tie %req, 'Tie::IxHash', $self->_revmap_fields(
-              content       => $content,
-              orderId       => 'invoice_number',
-              amount        => 'amount',
-              orderSource   => 'orderSource',
-              billToAddress => \%billToAddress,
-              card          => $content->{'card_number'} ? \%card : {},
-              token         => $content->{'card_token'} ? \%token : {},
-              customBilling => \%custombilling,
+              content                => $content,
+              orderId                => 'invoice_number',
+              amount                 => 'amount',
+              orderSource            => 'orderSource',
+              billToAddress          => \%billToAddress,
+              card                   => $content->{'card_number'} ? \%card : {},
+              token                  => $content->{'card_token'} ? \%token : {},
+              customBilling          => \%custombilling,
+              taxType                => 'tax_type',
+              enhancedData           => \%enhanceddata,
               processingInstructions => \%processing,
+              pos                    => \%pos,
+              amexAggregatorData     => \%amexaggregator,
+              merchantData           => \%merchantdata,
+              actionReason           => 'action_reason', # ENUM(SUSPECT_FRAUD) only option atm
           );
        }
     }
@@ -648,27 +836,28 @@ sub map_request {
         push @required_fields, qw( order_number );
         tie %req, 'Tie::IxHash',
           $self->_revmap_fields(
-            content                 => $content,
-            litleTxnId              => 'order_number',
-            processingInstructions  =>  \%processing,
+            content                  => $content,
+            litleTxnId               => 'order_number',
+            processingInstructions   => \%processing,
           );
     }
     elsif ( $action eq 'authReversal' ) {
         push @required_fields, qw( order_number amount );
         tie %req, 'Tie::IxHash',
           $self->_revmap_fields(
-            content    => $content,
-            litleTxnId => 'order_number',
-            amount     => 'amount',
+            content                  => $content,
+            litleTxnId               => 'order_number',
+            amount                   => 'amount',
+            actionReason             => 'action_reason', # ENUM(SUSPECT_FRAUD) only option atm
           );
     }
     elsif ( $action eq 'accountUpdate' ) {
         push @required_fields, qw( card_number expiration );
         tie %req, 'Tie::IxHash',
           $self->_revmap_fields(
-            content => $content,
-            orderId => 'customer_id',
-            card    => \%card,
+            content                  => $content,
+            orderId                  => 'customer_id',
+            card                     => \%card,
           );
     }
 
@@ -689,7 +878,7 @@ sub submit {
     warn 'Post processing: '.Dumper(\%content) if $DEBUG;
     my $post_data;
 
-    my $writer = new XML::Writer(
+    my $writer = XML::Writer->new(
         OUTPUT      => \$post_data,
         DATA_MODE   => 1,
         DATA_INDENT => 2,
@@ -1041,7 +1230,7 @@ sub create_batch {
 
     my $post_data;
 
-    my $writer = new XML::Writer(
+    my $writer = XML::Writer(
         OUTPUT      => \$post_data,
         DATA_MODE   => 1,
         DATA_INDENT => 2,
@@ -1180,7 +1369,7 @@ sub send_rfr {
     $self->_litle_init($args);
 
     my $post_data;
-    my $writer = new XML::Writer(
+    my $writer =  XML::Writer->new(
         OUTPUT      => \$post_data,
         DATA_MODE   => 1,
         DATA_INDENT => 2,
@@ -1406,7 +1595,7 @@ sub _revmap_fields {
         %content = %{ delete( $map{'content'} ) };
     }
     else {
-        warn "WARNING: This content has not been pre-processed with map_fields";
+        warn "WARNING: This content has not been pre-processed with map_fields ";
         %content = $self->content();
     }
 
@@ -1522,7 +1711,7 @@ sub chargeback_activity_request {
         $financials = 'false';
     }
 
-    my $writer = new XML::Writer(
+    my $writer = XML::Writer->new(
         OUTPUT      => \$post_data,
         DATA_MODE   => 1,
         DATA_INDENT => 2,
@@ -1634,7 +1823,7 @@ sub chargeback_update_request {
       croak "Missing arg $key" unless $content{$key};
     }
 
-    my $writer = new XML::Writer(
+    my $writer = XML::Writer->new(
         OUTPUT      => \$post_data,
         DATA_MODE   => 1,
         DATA_INDENT => 2,
@@ -1736,7 +1925,7 @@ Business::OnlinePayment::Litle - Business::OnlinePayment::Litle - Vantiv (was Li
 
 =head1 VERSION
 
-version 0.957
+version 0.958
 
 =head1 SYNOPSIS
 
@@ -1829,6 +2018,7 @@ The following actions are valid
   post authorization
   credit
   void
+  auth reversal
 
 =head2 Fields
 
@@ -2105,6 +2295,8 @@ Return a arrayref that contains a list of Business::OnlinePayment::Litle::Charge
  $tx->phoenixTxnId(); # Unique identifier provided by Litle.
  $tx->is_success(); # Boolean, did the request work
 
+=for html <a href="https://travis-ci.org/Jayceh/Business--OnlinePayment--Litle"><img src="https://travis-ci.org/Jayceh/Business--OnlinePayment--Litle.svg?branch=master"></a>
+
 =head1 METHODS AND FUNCTIONS
 
 See L<Business::OnlinePayment> for the complete list. The following methods either override the methods in L<Business::OnlinePayment> or provide additional functions.
@@ -2115,7 +2307,7 @@ See L<Business::OnlinePayment> for the complete list. The following methods eith
 
 =head1 SPECS
 
-Currently uses the Litle XML specifications version 8.12 and chargeback version 2.2
+Currently uses the Litle XML specifications version 11.0 and chargeback version 2.2
 
 =head1 TESTING
 
@@ -2144,10 +2336,11 @@ number as the first parameter and should return the masked version.
 
 Certain features are not yet implemented (no current personal business need), though the capability of support is there, and the test data for the verification suite is there.
 
-    Force Capture
     Capture Given Auth
-    3DS
-    billMeLater
+    applepay
+    paypage
+
+    return objects for bounce pages (sepa|ideal)
 
 =head1 BUGS
 
@@ -2166,6 +2359,10 @@ You can also look for information at:
 
 L<http://www.vantiv.com/>
 
+=head1 SPEC
+
+Documentation and specs are available on github at L<http://litleco.github.io/>
+
 =over 4
 
 =item * AnnoCPAN: Annotated CPAN documentation
